!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine readreleasesB(error)
!                               o
!*******************************************************************************
!                                                                              *
!     This routine reads the release point specifications for the current      *
!     model run. Several release points can be used at the same time.          *
!                                                                              *
!     Author: A. Stohl                                                         *
!                                                                              *
!     18 May 1996                                                              *
!
!     Changes: B. Legras, Apr. 2002
!              Release along a profile
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! decay               decay constant of species                                *
! dquer [um]          mean particle diameters                                  *
! dsigma              e.g. dsigma=10 or dsigma=0.1 means that 68% of the mass  *
!                     are between 0.1*dquer and 10*dquer                       *
! ireleasestart, ireleaseend [s] starting time and ending time of each release *
! npart               number of particles to be released                       *
! nspec               number of species to be released                         *
! density [kg/m3]     density of the particles                                 *
! rm [s/m]            Mesophyll resistance                                     *
! species             name of species                                          *
! xmass               total mass of each species                               *
! xpoint1,ypoint1     geograf. coordinates of lower left corner of release area*
! xpoint2,ypoint2     geograf. coordinates of upper right corner of release are*
! weta, wetb          parameters to determine the wet scavenging coefficient   *
! zpoint1,zpoint2     height range, over which release takes place             *
!                                                                              *
!*******************************************************************************

      use commons
      implicit none

      integer numpartmax,i,j,id1,it1,id2,it2
      real vsh(ni),fracth(ni),schmih(ni)
      double precision jul1,jul2,juldate
      character(len=50):: line
      logical, intent(out):: error
      logical old

      DEP=.false.
      DRYDEP=.false.
      WETDEP=.false.
      do i=1,maxspec
        DRYDEPSPEC(i)=.false.
      enddo

! Open the releases file and read user options
!*********************************************

      error=.false.

      open(unitreleases,file=path(1)(1:len(1))//'RELEASES',status='old',&
        err=999)


! Check the format of the RELEASES file (either in free format,
! or using formatted mask)
! Use of formatted mask is assumed if line 10 contains the word 'DIRECTION'
!**************************************************************************

      call skplin(12,unitreleases)
      read (unitreleases,901) line
901   format (a)
      if (index(line,'Total') .eq. 0) then
        old = .false.
      else
        old = .true.
      endif
      rewind(unitreleases)


! Skip first 11 lines (file header)
!**********************************

      call skplin(11,unitreleases)


! Read the number of species and the link to the species information table
! Assign species specific parameters needed for physical processes
!*************************************************************************

      read(unitreleases,*,err=998) nspec
      nspec = 1   ! imposed
      if (nspec.gt.maxspec) goto 994
      if (old) call skplin(2,unitreleases)
      do i=1,nspec        
        read(unitreleases,*,err=998) link(i)
        link(i)=1 ! imposed
        if (old) call skplin(2,unitreleases)
        species(i)=specname(link(i))

! Molecular weight
!*****************

        weightmolar(i)=weightmol(link(i))
        if (((iout.eq.2).or.(iout.eq.3)).and.  &
        (weightmolar(i).lt.0.)) then
          write(*,*) 'For mixing ratio output, valid molar weight'
          write(*,*) 'must be specified for all simulated species.'
          write(*,*) 'Check table SPECIES or choose concentration'
          write(*,*) 'output.'
          error=.true.
          return
        endif

! Radioactive decay
!******************

        decay(i)=0.693147/decaytime(link(i)) !conversion half life to decay constant

! Wet deposition
!***************

        weta(i)=wetscava(link(i))
        wetb(i)=wetscavb(link(i))

! Dry deposition of gases
!************************

        reldiff(i)=drydiff(link(i))             ! Diffusivity rel. to H20
        henry(i)=dryhenry(link(i))              ! Henry constant
        f0(i)=dryactiv(link(i))                 ! activity
        if (reldiff(i).gt.0.)  &
          rm(i)=1./(henry(i)/3000.+100.*f0(i))    ! mesophyll resistance

! Dry deposition of particles
!****************************

        vsetaver(i)=0.
        density(i)=partrho(link(i))                 ! Particle density
        dquer(i)=partmean(link(i))*1000000.     ! Conversion m to um
        dsigma(i)=partsig(link(i))
        if (density(i).gt.0.) then                  ! Additional parameters
          call part0(dquer(i),dsigma(i),density(i),ni,fracth,schmih,vsh)
          do j=1,ni
            fract(i,j)=fracth(j)
            schmi(i,j)=schmih(j)
            vset(i,j)=vsh(j)
            vsetaver(i)=vsetaver(i)-vset(i,j)*fract(i,j)
          enddo
        endif

! Dry deposition for constant deposition velocity
!************************************************

        dryvel(i)=dryvelo(link(i))*0.01         ! conversion to m/s

        if (weta(i).gt.0.) WETDEP=.true.
        if ((reldiff(i).gt.0.).or.(density(i).gt.0.).or. &
          (dryvel(i).gt.0.)) then
          DRYDEP=.true.
          DRYDEPSPEC(i)=.true.
        endif
      enddo

        if (WETDEP.or.DRYDEP) DEP=.true.
      read(unitreleases,*,err=998)


! Read specifications for each release point
!*******************************************

      numpoint=0
      numpartmax=0
100   numpoint=numpoint+1
      if (numpoint.gt.maxpoint) goto 997

      read(unitreleases,*,err=998,end=25) id1,it1
      if (old) call skplin(2,unitreleases)
      print *,'readreleases> ',id1,it1
!      read(unitreleases,*,err=998) id2,it2
!      if (old) call skplin(2,unitreleases)
      id2 = id1; it2 = it1  ! imposed
      read(unitreleases,*,err=998) xpoint1(numpoint)
      if (old) call skplin(2,unitreleases)
      read(unitreleases,*,err=998) ypoint1(numpoint)
      if (old) call skplin(2,unitreleases)
      print *,'readreleases> ',xpoint1(1),ypoint1(1)
      read(unitreleases,*,err=998) xpoint2(numpoint)
      if (old) call skplin(2,unitreleases)
      read(unitreleases,*,err=998) ypoint2(numpoint)
      if (old) call skplin(2,unitreleases)
      xpoint2(numpoint)=xpoint1(numpoint)  ! imposed
      ypoint2(numpoint)=ypoint1(numpoint)  ! imposed
      read(unitreleases,*,err=998) zpoint1(numpoint)
      if (old) call skplin(2,unitreleases)
      read(unitreleases,*,err=998) zpoint2(numpoint)
      if (old) call skplin(2,unitreleases)
      print *,'readreleases> ',zpoint1(1),zpoint2(1)
      read(unitreleases,*,err=998) numlevel(numpoint),nsample(numpoint)
      npart(numpoint) = numlevel(numpoint) * nsample(numpoint)
      if (old) call skplin(2,unitreleases)
      read(unitreleases,*,err=998) diffus
      if (old) call skplin(2,unitreleases)
      do i=1,nspec
        read(unitreleases,*,err=998) xmass(numpoint,i)
        if (old) call skplin(2,unitreleases)
      enddo
      read(unitreleases,'(a40)',err=998) compoint(numpoint)(1:40)
      if (old) call skplin(1,unitreleases)
      read(unitreleases,*,end=25)
      if((xpoint1(numpoint).eq.0.).and.(ypoint1(numpoint).eq.0.).and. &
         (xpoint2(numpoint).eq.0.).and.(ypoint2(numpoint).eq.0.).and. &
         (compoint(numpoint)(1:8).eq.'        ')) goto 25


! If a release point contains no particles, stop and issue error message
!***********************************************************************

      if (npart(numpoint).eq.0) then
        write(*,*) 'FLEXPART MODEL ERROR'
        write(*,*) 'RELEASES file is corrupt.'
        write(*,*) 'At least for one release point, there are zero'
        write(*,*) 'particles released. Make changes to RELEASES.'
        error=.true.
        return
      endif

! Determine relative beginning and ending times of particle release
!******************************************************************

      jul1=juldate(id1,it1)
      jul2=juldate(id2,it2)
      if (jul1.gt.jul2) then
        write(*,*) 'FLEXPART MODEL ERROR'
        write(*,*) 'Release stops before it begins.'
        write(*,*) 'Make changes to file RELEASES.'
        error=.true.
        return
      endif
      if (ldirect.eq.1) then
        if ((jul1.lt.bdate).or.(jul2.gt.edate)) then
          write(*,*) 'FLEXPART MODEL ERROR'
          write(*,*) 'Release starts before simulation begins or ends'
          write(*,*) 'after simulation stops.'
          write(*,*) 'Make files COMMAND and RELEASES consistent.'
          error=.true.
          return
        endif
        ireleasestart(numpoint)=int((jul1-bdate)*86400.)
        ireleaseend(numpoint)=int((jul2-bdate)*86400.)
      else if (ldirect.eq.-1) then
        if ((jul1.lt.edate).or.(jul2.gt.bdate)) then
          write(*,*) 'FLEXPART MODEL ERROR'
          write(*,*) 'Release starts before simulation begins or ends'
          write(*,*) 'after simulation stops.'
          write(*,*) 'Make files COMMAND and RELEASES consistent.'
          error=.true.
          return
        endif
        ireleasestart(numpoint)=int((jul1-bdate)*86400.)
        ireleaseend(numpoint)=int((jul2-bdate)*86400.)
      endif


! Check, whether the total number of particles does exceed totally allowed
! number of particles.
!*************************************************************************

      numpartmax=numpartmax+npart(numpoint)
      if (numpartmax.gt.maxpart) goto 996

      goto 100

25    close(unitreleases)
      numpoint=numpoint-1
      return


994   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - MAXIMUM NUMBER OF EMITTED SPECIES IS####'
      write(*,*) '#### TOO LARGE. PLEASE REDUCE NUMBER OF SPECIES. ####'
      write(*,*) '#####################################################'
      return


996   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - TOTAL NUMBER OF PARTICLES SPECIFIED ####'
      write(*,*) '#### IN FILE "RELEASES" EXCEEDS THE MAXIMUM      ####'
      write(*,*) '#### ALLOWED NUMBER. REDUCE EITHER NUMBER OF     ####'
      write(*,*) '#### PARTICLES PER RELEASE POINT OR REDUCE NUMBER####'
      write(*,*) '#### OF RELEASE POINTS.                          ####'
      write(*,*) '#####################################################'
      return


997   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### ERROR - NUMBER OF RELEASE POINTS SPECIFIED  ####'
      write(*,*) '#### IN FILE "RELEASES" EXCEEDS THE MAXIMUM      ####'
      write(*,*) '#### ALLOWED NUMBER.                             ####'
      write(*,*) '#####################################################'
      return


998   error=.true.
      write(*,*) '#####################################################'
      write(*,*) '#### FLEXPART MODEL SUBROUTINE READRELEASES:     ####'
      write(*,*) '####                                             ####'
      write(*,*) '#### FATAL ERROR - FILE "RELEASES" IS            ####'
      write(*,*) '#### CORRUPT. PLEASE CHECK YOUR INPUTS FOR       ####'
      write(*,*) '#### MISTAKES OR GET A NEW "RELEASES"-           ####'
      write(*,*) '#### FILE ...                                    ####'
      write(*,*) '#####################################################'
      return


999   error=.true.
      write(*,*)  
      write(*,*) '#####################################################'
      write(*,*) '   FLEXPART MODEL SUBROUTINE READRELEASES: '
      write(*,*)
      write(*,*) 'FATAL ERROR - FILE CONTAINING PARTICLE RELEASE POINTS'
      write(*,*) 'POINTS IS NOT AVAILABLE OR YOU ARE NOT'
      write(*,*) 'PERMITTED FOR ANY ACCESS'
      write(*,*) '#####################################################'
      return

      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
