!$Id:
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!     ##################################################################
!     ##################################################################
!     ######                                                      ######
!     ######                     Developed by                     ######
!     ######     Center for Analysis and Prediction of Storms     ######
!     ######                University of Oklahoma                ######
!     ######                                                      ######
!     ##################################################################
!     ##################################################################

      FUNCTION F_QVSAT( p, t ) 

!     PURPOSE:
!
!     Calculate the saturation specific humidity using enhanced Teten's
!     formula.
!
!     AUTHOR: Yuhe Liu
!     01/08/1998
!
!     MODIFICATION HISTORY:
!
!     INPUT :
!       p        Pressure (Pascal)
!       t        Temperature (K)
!     OUTPUT:
!       f_qvsat  Saturation water vapor specific humidity (kg/kg).
!
!     Variable Declarations.
!
      implicit none
 
      real p         ! Pressure (Pascal)
      real t         ! Temperature (K)
      real f_qvsat   ! Saturation water vapor specific humidity (kg/kg)
      real f_es,fespt
      
      real rd        ! Gas constant for dry air  (m**2/(s**2*K))
      parameter( rd     = 287.0 )
      real rv        ! Gas constant for water vapor  (m**2/(s**2*K)).
      parameter( rv     = 461.0 )
      real rddrv    
      parameter( rddrv  = rd/rv )


! Change by A. Stohl to save computation time:
      fespt=f_es(p,t)
      f_qvsat = rddrv * fespt / (p-(1.0-rddrv)*fespt)

      RETURN
      END

      FUNCTION F_ES( p, t ) 
!#######################################################################
!
!     PURPOSE:
!
!     Calculate the saturation specific humidity using enhanced Teten's
!     formula.
!
!     AUTHOR: Yuhe Liu
!     01/08/1998
!
!     MODIFICATION HISTORY:
!
!#######################################################################
!     INPUT :
!       p        Pressure (Pascal)
!       t        Temperature (K)
!     OUTPUT:
!       f_es     Saturation water vapor pressure (Pa)
!#######################################################################
!     Variable Declarations.

      implicit none
 
      real p         ! Pressure (Pascal)
      real t         ! Temperature (K)
      real f_es      ! Saturation water vapor pressure (Pa)
      real f_esl, f_esi

!      IF ( t.ge.273.15 ) THEN      ! for water
      IF ( t.ge.253.15 ) THEN      ! modification Petra Seibert
                                   ! (supercooled water may be present)
        f_es = f_esl( p,t )
      ELSE                            ! for ice
        f_es = f_esi( p,t )
      ENDIF

      RETURN
      END

      FUNCTION F_ESL( p, t )  

      implicit none
 
      real p         ! Pressure (Pascal)
      real t         ! Temperature (K)
      real f_esl     ! Saturation water vapor pressure over liquid water

      real f

!#######################################################################
!
!     Saturation specific humidity parameters used in enhanced Teten's
!     formula. (See A. Buck, JAM 1981)
!
!#######################################################################

      real satfwa, satfwb
      parameter ( satfwa = 1.0007 )
      parameter ( satfwb = 3.46e-8 )  ! for p in Pa

      real satewa, satewb, satewc
      parameter ( satewa = 611.21 )   ! es in Pa
      parameter ( satewb = 17.502 )
      parameter ( satewc = 32.18 )

      real satfia, satfib
      parameter ( satfia = 1.0003 )
      parameter ( satfib = 4.18e-8 )  ! for p in Pa

      real sateia, sateib, sateic
      parameter ( sateia = 611.15 )   ! es in Pa
      parameter ( sateib = 22.452 )
      parameter ( sateic = 0.6 )

      f = satfwa + satfwb * p
      f_esl = f * satewa * exp( satewb*(t-273.15)/(t-satewc) )

      RETURN
      END

      FUNCTION F_ESI( p, t )  

      implicit none
 
      real p         ! Pressure (Pascal)
      real t         ! Temperature (K)
      real f_esi     ! Saturation water vapor pressure over ice (Pa)

      real f

!#######################################################################
!
!     Saturation specific humidity parameters used in enhanced Teten's
!     formula. (See A. Buck, JAM 1981)
!
!#######################################################################
!
      real satfwa, satfwb
      parameter ( satfwa = 1.0007 )
      parameter ( satfwb = 3.46e-8 )  ! for p in Pa

      real satewa, satewb, satewc
      parameter ( satewa = 611.21 )   ! es in Pa
      parameter ( satewb = 17.502 )
      parameter ( satewc = 32.18 )

      real satfia, satfib
      parameter ( satfia = 1.0003 )
      parameter ( satfib = 4.18e-8 )  ! for p in Pa

      real sateia, sateib, sateic
      parameter ( sateia = 611.15 )   ! es in Pa
      parameter ( sateib = 22.452 )
      parameter ( sateic = 0.6 )

      f = satfia + satfib * p
      f_esi = f * sateia * exp( sateib*(t-273.15)/(t-sateic) )

      RETURN
      END
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
!
!
