!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine readavailable(error)
!                                o
!*******************************************************************************
!                                                                              *
!   This routine reads the dates and times for which windfields are available. *
!                                                                              *
!     Authors: A. Stohl                                                        *
!                                                                              *
!     6 February 1994                                                          *
!     8 February 1999, Use of nested fields, A. Stohl    
!     7 April    2002, Suppression of code related to nested fields, B. Legras
!                                                                              *
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! bdate                beginning date as Julian date                           *
! beg                  beginning date for windfields                           *
! end                  ending date for windfields                              *
! error                .true., if error ocurred in subprogram, else .false.    *
! fname                filename of wind field, help variable                   *
! ideltas [s]          duration of modelling period                            *
! idiff                time difference between 2 wind fields                   *
! idiffnorm            normal time difference between 2 wind fields            *
! idiffmax [s]         maximum allowable time between 2 wind fields            *
! jul                  julian date, help variable                              *
! numbwf               actual number of wind fields                            *
! wfname(maxwf)        file names of needed wind fields                        *
! wfspec(maxwf)        file specifications of wind fields (e.g., if on disc)   *
! wftime(maxwf) [s]times of wind fields relative to beginning time             *
! wfname1,wfspec1,wftime1 = same as above, but only local (help variables)     *
!                                                                              *
! Constants:                                                                   *
! maxwf                maximum number of wind fields                           *
! unitavailab          unit connected to file AVAILABLE                        *
!                                                                              *
!*******************************************************************************

      use commons
      implicit none

      logical error,toobig
      integer i,j,idiff,ldat,ltim
      integer year, month, day
      double precision juldate,jul,beg,end
      character(len=16):: fname,spec
      character(len=16), allocatable:: wfname1(:),wfspec1(:)
      integer, allocatable:: wfldat1(:),wfltim1(:),wftime1(:)

      error=.false.
      allocate(wfname1(maxwf),wfspec1(maxwf), &
               wfldat1(maxwf),wfltim1(maxwf),wftime1(maxwf))

! Windfields are only used, if they are within the modelling period.
! However, 1 additional day at the beginning and at the end is used for
! interpolation. -> Compute beginning and ending date for the windfields.
!************************************************************************

      if (ideltas.gt.0) then         ! forward trajectories
        beg=bdate-1.                 ! idiffmax should be used here too               
        end=bdate+dble(float(ideltas)/86400.) &
                 +dble(float(idiffmax)/86400.)
      else                           ! backward trajectories
        beg=bdate+dble(float(ideltas)/86400.) &
                 -dble(float(idiffmax)/86400.)
        end=bdate+1.                 ! idiffmax shoukd be used here too
      endif

! Open the wind field availability file and read available wind fields
! within the modelling period.
!*********************************************************************

      open(unitavailab,file=path(4)(1:len(4)),status='old',err=999)

      do i=1,3
        read(unitavailab,*)
      enddo
      
      numbwf=0
100   read(unitavailab,'(i8,1x,i6,3x,a16,3x,a10)',end=99) &
           ldat,ltim,fname,spec
      jul=juldate(ldat,ltim)
      if ((jul.ge.beg).and.(jul.le.end)) then
        numbwf=numbwf+1
        if (numbwf.gt.maxwf) then      ! check exceedance of dimension
           write(*,*) 'Number of needed wind fields is too great.'
           write(*,*) 'actual, max ',numbwf,maxwf
           write(*,*) 'Reduce modelling period (file "COMMAND") or'
           write(*,*) 'reduce number of wind fields (file "AVAILABLE").'
           goto 1000
        endif

        wfname1(numbwf)=adjustl(fname)
        wfldat1(numbwf)=ldat
        wfltim1(numbwf)=ltim
        wfspec1(numbwf)=trim(spec)
        wftime1(numbwf)=nint((jul-bdate)*86400.)
      endif
      goto 100       ! next wind field

99    continue
      write(*,*) 'readavailable> numbwf',numbwf,maxwf
      write(*,*) 'readavailable> min, max ',&
        minval(wftime1(1:numbwf)),maxval(wftime1(1:numbwf))

      close(unitavailab)


! Check wind field times of file AVAILABLE (expected to be in temporal order)
!****************************************************************************

      if (numbwf.eq.0) then
        write(*,*) ' #### FLEXPART MODEL ERROR! NO WIND FIELDS    #### '
        write(*,*) ' #### AVAILABLE FOR SELECTED TIME PERIOD.     #### '
        error=.TRUE.
        return
      endif

      do i=2,numbwf
        if (wftime1(i).le.wftime1(i-1)) then
          write(*,*) 'FLEXPART ERROR: FILE AVAILABLE IS CORRUPT.'
          write(*,*) 'THE WIND FIELDS ARE NOT IN TEMPORAL ORDER.'
          write(*,*) 'PLEASE CHECK FIELD ',wfname1(i)
          error=.TRUE.
          return
        endif
      enddo


! For backward trajectories, reverse the order of the windfields
!***************************************************************

      if (ideltas.ge.0) then
        wfname(1:numbwf)=wfname1(1:numbwf)
        wfspec(1:numbwf)=wfspec1(1:numbwf)
        wftime(1:numbwf)=wftime1(1:numbwf)
        wfldat(1:numbwf)=wfldat1(1:numbwf)
        wfltim(1:numbwf)=wfltim1(1:numbwf)
      else
        do i=1,numbwf
          wfname(numbwf-i+1)=wfname1(i)
          wfspec(numbwf-i+1)=wfspec1(i)
          wftime(numbwf-i+1)=wftime1(i)
          wfldat(numbwf-i+1)=wfldat1(i)
          wfltim(numbwf-i+1)=wfltim1(i)
        enddo
      endif

! Check the time difference between the wind fields. If it is big, 
! write a warning message. If it is too big, warn that trajectory will
! terminate. 
!*********************************************************************

      wftime1(:)=wftime(:)
      do i=2,numbwf
        idiff=abs(wftime(i)-wftime(i-1))
        toobig=.false.
        if (idiff.gt.idiffmax) then
!         detect end of february for bissextil year in perpetual run
!         shift all remaining times in such case to satisfy check done in
!         getfield
          if(perpetual) then
            ! check done on unshifted times to match successive bissextil years 
            if(ideltas > 0) call caldate(wftime1(i-1)/86400.+bdate,ldat,ltim)
            if(ideltas < 0) call caldate(wftime1(i  )/86400.+bdate,ldat,ltim)
            year=int(ldat/10000)
            month=int((ldat-10000*year)/100)
            day=int(ldat-10000*year-100*month)
            if(month==2.and.day==28.and.mod(year,4)==0) then
              print *,'readavailable > jump due to bissextil year in perpetual run'
              print *,'readavailable > apply one-day shift to all remaining times'
              if(ideltas > 0) then
                do j=i,numbwf
                  wftime(j)=wftime(j)-86400
                enddo
              else
                do j=i,numbwf
                  wftime(j)=wftime(j)+86400
                enddo
              endif
            else
              toobig=.true.
            endif
          endif
          if(.not.perpetual.or.toobig) then
            write(*,*) 'FLEXPART WARNING: TIME DIFFERENCE BETWEEN TWO'
            write(*,*) 'WIND FIELDS IS TOO BIG FOR TRANSPORT CALCULATION.'
            write(*,*) 'THEREFORE, TRAJECTORIES HAVE TO BE SKIPPED.'
            print *, wfname(i),wfname(i-1),idiffmax
          endif
        else if (idiff.gt.idiffnorm) then
          write(*,*) 'FLEXPART WARNING: TIME DIFFERENCE BETWEEN TWO'
          write(*,*) 'WIND FIELDS IS BIG. THIS MAY CAUSE A DEGRADATION'
          write(*,*) 'OF SIMULATION QUALITY.'
        endif
      enddo


! Reset the times of the wind fields that are kept in memory to no time
!**********************************************************************

      do i=1,2
        memind(i)=i
        memtime(i)=999999999
      enddo

      deallocate(wfname1,wfspec1,wfldat1,wftime1,wfltim1)

      return    

999   write(*,*) ' #### FLEXPART MODEL ERROR! FILE #### '
      write(*,'(a)') '     '//path(4)(1:len(4)) 
      write(*,*) ' #### CANNOT BE OPENED           #### '
1000  error=.true.

      return
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log: 
