!$Id: 
!
!=======================================================================
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ FLEXPART @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!=====|==1=========2=========3=========4=========5=========6=========7==
!
      subroutine timemanagerB()                                  

!*******************************************************************************
!                                                                              *
! Handles the computation of trajectories, i.e. determines which               *
! trajectories have to be computed at what time.                               *
! Manages dry+wet deposition routines, radioactive decay and the computation   *
! of concentrations.                                                           *
!                                                                              *
!     Author: A. Stohl                                                         *
!                                                                              *
!     20 May 1996                                                              *
!                                                                              *
!*******************************************************************************
!  Changes, Bernd C. Krueger, Feb. 2001:
!           Call of convmix when new windfield is read
!           B. Legras, Apr. 2002:
!           simplified version for stratospheric studies
!           (...)
!           B. Legras, March 2006:
!           delayed initialization
!           B. Legras, September 2012
!           AGEF mode
!           // version
!           B. Legras 16/9/2012
!           stopped trajs removed from npproc 
!*******************************************************************************
!                                                                              *
! Variables:                                                                   *
! DEP                .true. if either wet or dry deposition is switched on     *
! decay(maxspec) [1/s] decay constant for radioactive decay                    *
! DRYDEP             .true. if dry deposition is switched on                   *
! ideltas [s]        modelling period                                          *
! itime [s]          actual temporal position of calculation                   *
! ldeltat [s]        time since computation of radioact. decay of depositions  *
! loutaver [s]       averaging period for concentration calculations           *
! loutend [s]        end of averaging for concentration calculations           *
! loutnext [s]       next time at which output fields shall be centered        *
! loutsample [s]     sampling interval for averaging of concentrations         *
! loutstart [s]      start of averaging for concentration calculations         *
! loutstep [s]       time interval for which concentrations shall be calculated*
! npoint(maxpart)    index, which starting point the trajectory has            *
!                    starting positions of trajectories                        *
! outnum             number of samples for each concentration calculation      *
! prob               probability of absorption at ground due to dry deposition *
! WETDEP             .true. if wet deposition is switched on                   *
! weight             weight for each concentration sample (1/2 or 1)           *
! uap(maxpart),ucp(maxpart),uzp(maxpart) = random velocities due to turbulence *
! us(maxpart),vs(maxpart),ws(maxpart) = random velocities due to interpolation *
! xtra1(maxpart), ytra1(maxpart), ztra1(maxpart) =                             *
!                    spatial positions of trajectories                         *
!                                                                              *
! Constants:                                                                   *
! maxpart            maximum number of trajectories                            *
!                                                                              *
!*******************************************************************************

      use commons
      use lyapunov, only: activ_Lyapunov, numpart_np, tau_orthog, &
                          lambdaa, lambdab, &
                          ortho, &
                          mass_triad_b  !test----
      use ecmwf_diab
      use ecmwf_inct
      use mass_iso
      use combin
      implicit none

      integer i,j,k,l,n,itime,nstop
      integer loutnext,loutnext0,loutstart,loutend
      integer ix,jy,kz,ldeltat,itage,nage,npproc,npstop(7),nb_bounce
      integer count_clock_1,count_clock_2,count_rate,count_max
      integer next_time_orthog, prev_time_orthog
      integer, allocatable :: shuffle(:)
      integer npproc_thread,nb_bounce_thread,npstop_thread(7),num_thread
      real cpu_cost
      real outnum,weight,prob(maxspec)
      real tint  
      real xold,yold,zold,dt
      real satratio
!     real lambdaa(3,maxlyap), lambdab(3,maxlyap)
!      real xm,xm1
      logical error 
      logical, allocatable :: mask_part(:)
      integer OMP_GET_THREAD_NUM
!     external ctrper

      !test-----
      real cc
      !---------

      if (shuffling) then
! Generate a random permutation among the numpart elements
! needs numpart to be defined (no delayed initialisation)
         print *,'shuffling over numpart ',numpart
         allocate(shuffle(numpart))
         call ranper(shuffle,.true.)
         print *,shuffle(1:5)
         print *,minval(shuffle),maxval(shuffle)
      endif
      
      if (TTLactiv) then
! Allocate arrays for temperature and saturation mixing ratio
        print *,'timemanager> allocate ttra1 and qtra1'
        if (.not.allocated(ttra1)) allocate(ttra1(maxpart))
        if (.not.allocated(qtra1)) allocate (qtra1(maxpart))
! mixing ratio read if restart but here it is reinitalized
! mixing ratio initialized at the beginning, after each output or after restart
        qtra1=0.05
      endif

      if (AGEFactiv) then
        if (.not.allocated(ttra1)) allocate(ttra1(maxpart))
      endif     

! Initialisation of the variables controling the output
      loutnext = loffset
!     loutnext0 allows first output out of sequence
!     and also to have output at time 0 if loffset2 = -loutnext
      loutnext0 = loutnext + loffset2
      
      if( activ_Lyapunov ) then
        print *,'timemanager> activation of Lyapunov calculation'
!       first orthogonalization performed at the same time as the first output
        prev_time_orthog = 0
        next_time_orthog = loutnext0
        !test----
        mass_triad_b(1:numpart_np) = exp(-ztra1(1:numpart_np))
        !--------
      endif

!**********************************************************************
! Loop over the whole modelling period in time steps of mintime seconds
!**********************************************************************

      npstop(:)=0
      npproc=numpart ! particle counter initialized to numpart (which may not
                     ! be defined at this stage)
      print *,'start time loop'  

      dotime: do itime=itime0,ideltas,lsynctime
  
!       print *,'timemanager> itime, loutnext, npproc', itime, loutnext0, npproc 

! Get necessary wind fields if not available
!*******************************************

        if(iso_mass) then
          call getfields_iso(itime,nstop)
        else
          call getfields(itime,nstop)
        endif
        if (nstop > 1) stop 'NO METEO FIELDS AVAILABLE'
        if(diabatic_w .and. ecmwf_diabatic_w) then
          call getfields_diab(itime,nstop)
          if (nstop > 1) stop 'NO DIAB FIELDS AVAILABLE'
        endif
        if(diabatic_w .and. ecmwf_inct_w) then
          call getfields_inct(itime,nstop)
          if (nstop > 1) stop 'NO INCT FIELDS AVAILABLE'
        endif

! Switch off the diffusion if needed 
        if ((delay_switch_diff_off > 0).and. &
              (-itime > delay_switch_diff_off)) then
           switch_diff_off=.true.
           print *,'switch_diff_off changed to true'
        endif

! Delayed initialization if required
        if(itime==itime0.and.delayed_initialization.and.(.not.restart)) then
          if(press2theta) then
            call fixpresslayerpart(error)
            if(error) stop 'ERROR in fixpresslayerpart for' &
                           //' delayed initialization'
          else if(theta2press) then 
            stop 'THETA2PRESS: OPTION NOT IMPLEMENTED'
          else if(make_uni3D) then ! case of CO2 runs
            call fixlayerpart(error)
            if(error) stop 'ERROR in fixlayerpart for' &
                           //' delayed initialization'
          else
            stop 'NO OPTION FOR DELAYED INITIALIZATION'
          endif
          if(TTLactiv) then
            qtra1=0.05
          endif
        endif

! Output of particle positions
         
        if((itime == loutnext0).and.(ipout == 1)) then
          if (TTLactiv) then
            call partout_qv(itime)
       !     print *, &
       !       'output of particle positions, T and qv, format 103 ', &
       !       itime/86400,' days'
            qtra1= 0.05  ! reinitialize qtra1 after each output 
          else if (AGEFactiv) then
            call partout_agef(itime)
       !     print *,'output of particle positions and T, format 105 ', &
       !       itime/86400,' days'
          else
            call partout_fast(itime)           !  dump particle position
       !     print *,'output of particle positions format 102 ', &
       !        itime/86400,' days' 
          endif  
          loutnext=loutnext+loutstep
          loutnext0 = loutnext
          if(savfull.or.(kind(1.0)==8)) then
            call savsav(itime)
            print *,'full backup of trajectories ',itime
          endif
          call flush(6)
        endif                                   ! interval, not at the end
 
! Calculate Lyapunov exponents: TO BE UPDATED WITH I. PISSO CODE
     
        if( activ_Lyapunov .and. (itime == next_time_orthog) ) then
           print *,'timemanager> Gram-Schmidt'
           call ortho(itime)
!	   if( prev_time_orthog == 0 ) then	  
!	      do i = 1, numpart_np
!	         lambda(:,i) = log / (itime-itramem(i))
!             enddo              
!	   else
!	      do i = 1, numpart_np
!	         lambda(:,i) = 0.5* (lambdaa(:,i)+lambdab(:,i)) / (itime - prev_time_orthog)
!	      enddo
!	   endif
           !test----
           do i=1,numpart_np
             cc = 1./(itime - itramem(i))   
             write(*,'(a,i8,i5,3G15.5)') ' Lyapunov> ',itime,i,  &
                cc*lambdaa(1,i),cc*lambdaa(2,i),cc*lambdaa(3,i)
             write(*,'(a,i8,i5,3G15.5)') ' Lyapunov> ',itime,i,  &
                cc*lambdab(1,i),cc*lambdab(2,i),cc*lambdab(3,i)
           enddo
           mass_triad_b(1:numpart_np) = exp(-ztra1(1:numpart_np))
           !--------
	   prev_time_orthog = itime
	   next_time_orthog = itime + tau_orthog
	   !call lambda_out(itime)
	endif
	   
        if (itime == ideltas) goto 99           ! do not advect beyond final time

!************************
! Loop over all particles
!************************
        npproc=0 ; nb_bounce=0
        call system_clock(count_clock_1,count_rate,count_max)

!$OMP PARALLEL DEFAULT(SHARED) SHARED(npproc,nb_bounce,npstop,shuffle,itime) &
!$OMP PRIVATE(nstop,npproc_thread,nb_bounce_thread,npstop_thread,num_thread)
        npproc_thread=0
        nb_bounce_thread=0
        npstop_thread(:)=0
        num_thread=OMP_GET_THREAD_NUM()
!$OMP DO SCHEDULE(RUNTIME) PRIVATE(i,j,dt,tint)
        dopart: do i=1,numpart
          if (shuffling) then
             j=shuffle(i)
          else
             j=i
          endif

        ! nstop now controls if an individual trajectory needs to be terminated
        ! set to 0 in advance       

        ! TEST
        !  if((j==1718).or.(j==4191).or.(j==31663).or.(j==35817) ) then
        !    debug_out=.true.
        !  else 
        !    debug_out=.false.
        !  endif
     
! Determine age class of the particle
!         itage=abs(itra1(j)-itramem(j))
!         do 36 nage=1,nageclass
!           if (itage.lt.lage(nage)) goto 37
!36          continue
!37        continue

! Terminate particles that are older than the maximum allowed age
!****************************************************************

!          if (itage.gt.lage(nageclass)) itra1(j)=-999999999

! If integration step is due, do it
!**********************************
          ifdue: if ((abs(itime+lsynctime)> abs(itra1(j))) .and. &
              (abs(itra1(j))>= abs(itime))) then
            if(itra1(j) == itime) then
               dt = float(abs(lsynctime))
            else
               dt = float(abs(itime+lsynctime-itra1(j)))
            endif

! Integrate advection equation for lsynctime seconds
!***************************************************
            if(debug_out) &
               print "(' timemanager> particle ',i5,2X,3G12.5)",&
                  j,xtra1(j),ytra1(j),ztra1(j)
            call advanceB(j,itime,dt,nstop, &
              xtra1(j),ytra1(j),ztra1(j),tint)     
            npproc_thread=npproc_thread+1
            !---- Temporary print
            !if (j <= numpart_np) then
            !  write(*,'(a,2i10,3g21.12)')' advance>',itime, j, &
            !      xlon0+xtra1(j)*dx,ylat0+ytra1(j)*dy,p0*exp(-ztra1(j))
            !endif
            !----
! Test ------------------------------
! may generate large amount of output
!cc         if(nstop.gt.1) print *,'stop traj ',j,xtra1(j),ytra1(j),ztra1(j)
!------------------------------------

! Calculate qv for final pressure and initial temperature
!********************************************************
            if(TTLactiv) then
              ttra1(j)=tint
              qtra1(j)=min(qtra1(j),satratio(p0*exp(-ztra1(j)),tint))
              !---- Temporary test
              !if(qtra1(j)==0.05) then
              !   print *,'qtra1 anomaly report'
              !   print *,j,tint,p0*exp(-ztra1(j))
              !endif
              !----
            else if (AGEFactiv) then
              ttra1(j)=tint
            endif
           
! Calculate the gross fluxes across layer interfaces
!***************************************************

!            if (iflux.eq.1) call calcfluxes(nage,j,xold,yold,zold)

! Determine, when next time step is due
! If trajectory is terminated, do not increment its time
!*******************************************************

            if (nstop>1) then           
              npstop_thread(nstop)=npstop_thread(nstop)+1             
!             TEST    
!              write(*,'(a,2i8,3f12.4)') ' timemanager > stopped parcel',&
!                 j,nstop,xtra1(j),ytra1(j),ztra1(j)
!             itra1(j)=-999999999
            elseif (nstop<0) then
              nb_bounce_thread=nb_bounce_thread+1
!             print *,'timemanager > bouncing parcel'
!             print *,j,nstop,xtra1(j),ytra1(j),ztra1(j)
              itra1(j)=itime+lsynctime
            else
              itra1(j)=itime+lsynctime
            endif

          endif ifdue
         
        enddo dopart                      ! end of loop over particles

!$OMP END DO

!$OMP CRITICAL
        npproc=npproc+npproc_thread
        nb_bounce=nb_bounce+nb_bounce_thread
        npstop(:)=npstop(:)+npstop_thread(:)
        npproc=npproc-sum(npstop_thread)
!$OMP END CRITICAL
!       print *,'THREAD ',num_thread,' completed'
!$OMP END PARALLEL

        if (mod(itime,96*lsynctime)==0) then
          call system_clock(count_clock_2,count_rate,count_max)
          write(*,'(a,i8,f10.2,a)') ' timemanager> npproc*, cpu ',&
             npproc, float(count_clock_2-count_clock_1)/       &
             float(count_rate), 's'
          write(*,'(a,7i8)') ' timemanager> npstop ',npstop
          allocate (mask_part(numpart))
          mask_part(:) = itra1(:)==itime+lsynctime
!        print*,'xtra1 ', minval(xtra1(1:numpart),DIM=1,MASK=mask_part), &
!                         maxval(xtra1(1:numpart),DIM=1,MASK=mask_part)
!        print*,'ytra1 ', minval(ytra1(1:numpart),DIM=1,MASK=mask_part), &
!                         maxval(ytra1(1:numpart),DIM=1,MASK=mask_part)
          print*,'ztra1 ', minval(ztra1(1:numpart),DIM=1,MASK=mask_part), &
                           maxval(ztra1(1:numpart),DIM=1,MASK=mask_part)
          if(iso_mass) then                 
             mask_part(:) = mask_part(:) .and. (ztra1(:) .ge. ThetaLev(NbTheta))
             print*,'# hanged ',sum(mask_part)
          endif
          deallocate (mask_part)
          npstop(:)=0
        endif
        if (nb_bounce >0) print *,'timemanager > nb bouncing ',nb_bounce
        
!        if(npstop.eq.1) stop
          
      enddo dotime                         ! end of loop over simulation time

      write(*,'(a,7i8)') 'timemanager > npstop ',npstop

99    continue

      if (ipout.eq.2) call partout_fast(itime)     ! dump particle positions
      if(shuffling) deallocate (shuffle)

      return
      end
!
!=====|==1=========2=========3=========4=========5=========6=========7==
!
!$Log:
